@@ -19,29 +19,39 @@
 # along with duplicity; if not, write to the Free Software Foundation,
 # Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 
-"""
+u"""
 Provides temporary file handling cenetered around a single top-level
 securely created temporary directory.
 
 The public interface of this module is thread-safe.
 """
 
-import os
-import threading
-import tempfile
+from __future__ import print_function
+from future import standard_library
+standard_library.install_aliases()
+from builtins import object
 
+import os
+import platform
+import subprocess
+import tempfile
+import threading
+
+from duplicity import globals
 from duplicity import log
 from duplicity import util
-from duplicity import globals
 
 # Set up state related to managing the default temporary directory
 # instance
 _defaultLock = threading.Lock()
 _defaultInstance = None
+# backup the initial tmp dir path because we will force tempfile
+# later to use our generated _defaultInstance.dir() as temproot
+_initialSystemTempRoot = tempfile.gettempdir()
 
 
 def default():
-    """
+    u"""
     Obtain the global default instance of TemporaryDirectory, creating
     it first if necessary. Failures are propagated to caller. Most
     callers are expected to use this function rather than
@@ -57,13 +67,15 @@ def default():
     try:
         if _defaultInstance is None or _defaultInstance.dir() is None:
             _defaultInstance = TemporaryDirectory(temproot=globals.temproot)
+            # set the temp dir to be the default in tempfile module from now on
+            tempfile.tempdir = _defaultInstance.dir()
         return _defaultInstance
     finally:
         _defaultLock.release()
 
 
-class TemporaryDirectory:
-    """
+class TemporaryDirectory(object):
+    u"""
     A temporary directory.
 
     An instance of this class is backed by a directory in the file
@@ -109,16 +121,43 @@ class TemporaryDirectory:
     'attackers' file.
     """
     def __init__(self, temproot=None):
-        """
+        u"""
         Create a new TemporaryDirectory backed by a unique and
         securely created file system directory.
 
         tempbase - The temp root directory, or None to use system
         default (recommended).
         """
-        self.__dir = tempfile.mkdtemp("-tempdir", "duplicity-", temproot)
+        def defaults_to_tmp(path):
+            u'''Determine if path point to a MAcOS system tmp'''
+            sys_temps = [
+                os.path.realpath(u"/tmp"),
+                os.path.realpath(u"/var/tmp"),
+            ]
 
-        log.Info(_("Using temporary directory %s") % util.ufn(self.__dir))
+            user_temp = os.path.realpath(path)
+            for sys_temp in sys_temps:
+                if user_temp.startswith(sys_temp):
+                    return True
+            return False
+
+        if temproot is None:
+            if globals.temproot:
+                temproot = globals.temproot
+            else:
+                global _initialSystemTempRoot
+                temproot = _initialSystemTempRoot
+        if isinstance(temproot, b"".__class__):
+            temproot = util.fsdecode(temproot)
+
+        if (platform.system().startswith(u'Darwin') and defaults_to_tmp(temproot)):
+            # Use temp space from getconf, never /tmp
+            temproot = subprocess.check_output([u'getconf', u'DARWIN_USER_TEMP_DIR'])
+            temproot = util.fsdecode(temproot).rstrip()
+
+        self.__dir = tempfile.mkdtemp(u"-tempdir", u"duplicity-", temproot)
+
+        log.Info(_(u"Using temporary directory %s") % self.__dir)
 
         # number of mktemp()/mkstemp() calls served so far
         self.__tempcount = 0
@@ -130,13 +169,13 @@ class TemporaryDirectory:
         self.__lock = threading.Lock()  # protect private resources *AND* mktemp/mkstemp calls
 
     def dir(self):
-        """
+        u"""
         Returns the absolute pathname of the temp folder.
         """
         return self.__dir
 
     def __del__(self):
-        """
+        u"""
         Perform cleanup.
         """
         global _defaultInstance
@@ -144,7 +183,7 @@ class TemporaryDirectory:
             self.cleanup()
 
     def mktemp(self):
-        """
+        u"""
         Return a unique filename suitable for use for a temporary
         file. The file is not created.
 
@@ -159,10 +198,10 @@ class TemporaryDirectory:
         self.__lock.acquire()
         try:
             self.__tempcount = self.__tempcount + 1
-            suffix = "-%d" % (self.__tempcount,)
-            filename = tempfile.mktemp(suffix, "mktemp-", self.__dir)
+            suffix = u"-%d" % (self.__tempcount,)
+            filename = util.fsencode(tempfile.mktemp(suffix, u"mktemp-", self.__dir))
 
-            log.Debug(_("Registering (mktemp) temporary file %s") % util.ufn(filename))
+            log.Debug(_(u"Registering (mktemp) temporary file %s") % util.fsdecode(filename))
             self.__pending[filename] = None
         finally:
             self.__lock.release()
@@ -170,7 +209,7 @@ class TemporaryDirectory:
         return filename
 
     def mkstemp(self):
-        """
+        u"""
         Returns a filedescriptor and a filename, as per os.mkstemp(),
         but located in the temporary directory and subject to tracking
         and automatic cleanup.
@@ -181,10 +220,10 @@ class TemporaryDirectory:
         self.__lock.acquire()
         try:
             self.__tempcount = self.__tempcount + 1
-            suffix = "-%d" % (self.__tempcount,)
-            fd, filename = tempfile.mkstemp(suffix, "mkstemp-", self.__dir)
+            suffix = u"-%d" % (self.__tempcount,)
+            fd, filename = tempfile.mkstemp(suffix, u"mkstemp-", self.__dir,)
 
-            log.Debug(_("Registering (mkstemp) temporary file %s") % util.ufn(filename))
+            log.Debug(_(u"Registering (mkstemp) temporary file %s") % filename)
             self.__pending[filename] = None
         finally:
             self.__lock.release()
@@ -192,16 +231,16 @@ class TemporaryDirectory:
         return fd, filename
 
     def mkstemp_file(self):
-        """
+        u"""
         Convenience wrapper around mkstemp(), with the file descriptor
         converted into a file object.
         """
         fd, filename = self.mkstemp()
 
-        return os.fdopen(fd, "r+"), filename
+        return os.fdopen(fd, u"r+"), filename
 
     def forget(self, fname):
-        """
+        u"""
         Forget about the given filename previously obtained through
         mktemp() or mkstemp(). This should be called *after* the file
         has been deleted, to stop a future cleanup() from trying to
@@ -216,16 +255,16 @@ class TemporaryDirectory:
         self.__lock.acquire()
         try:
             if fname in self.__pending:
-                log.Debug(_("Forgetting temporary file %s") % util.ufn(fname))
+                log.Debug(_(u"Forgetting temporary file %s") % util.fsdecode(fname))
                 del(self.__pending[fname])
             else:
-                log.Warn(_("Attempt to forget unknown tempfile %s - this is probably a bug.") % util.ufn(fname))
+                log.Warn(_(u"Attempt to forget unknown tempfile %s - this is probably a bug.") % util.fsdecode(fname))
                 pass
         finally:
             self.__lock.release()
 
     def cleanup(self):
-        """
+        u"""
         Cleanup any files created in the temporary directory (that
         have not been forgotten), and clean up the temporary directory
         itself.
@@ -236,17 +275,18 @@ class TemporaryDirectory:
         self.__lock.acquire()
         try:
             if self.__dir is not None:
-                for file in self.__pending.keys():
+                for file in list(self.__pending.keys()):
                     try:
-                        log.Debug(_("Removing still remembered temporary file %s") % util.ufn(file))
+                        log.Debug(_(u"Removing still remembered temporary file %s") % util.fsdecode(file))
                         util.ignore_missing(os.unlink, file)
                     except Exception:
-                        log.Info(_("Cleanup of temporary file %s failed") % util.ufn(file))
+                        log.Info(_(u"Cleanup of temporary file %s failed") % util.fsdecode(file))
                         pass
                 try:
                     os.rmdir(self.__dir)
                 except Exception:
-                    log.Warn(_("Cleanup of temporary directory %s failed - this is probably a bug.") % util.ufn(self.__dir))
+                    log.Warn(_(u"Cleanup of temporary directory %s failed - "
+                               u"this is probably a bug.") % util.fsdecode(self.__dir))
                     pass
                 self.__pending = None
                 self.__dir = None
