@@ -18,6 +18,10 @@
 # along with duplicity; if not, write to the Free Software Foundation,
 # Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 
+from __future__ import print_function
+from future import standard_library
+standard_library.install_aliases()
+
 import unittest
 from duplicity import cached_ops
 from duplicity import tarfile
@@ -29,5 +33,5 @@ class TarfileTest(UnitTestCase):
         self.assertTrue(tarfile.grp is cached_ops)
         self.assertTrue(tarfile.pwd is cached_ops)
 
-if __name__ == "__main__":
+if __name__ == u"__main__":
     unittest.main()
